# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Type, Iterable, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import (
    Body,
    Omit,
    Query,
    Headers,
    NotGiven,
    FileTypes,
    SequenceNotStr,
    omit,
    not_given,
)
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from .metadata_index import (
    MetadataIndexResource,
    AsyncMetadataIndexResource,
    MetadataIndexResourceWithRawResponse,
    AsyncMetadataIndexResourceWithRawResponse,
    MetadataIndexResourceWithStreamingResponse,
    AsyncMetadataIndexResourceWithStreamingResponse,
)
from ...._base_client import AsyncPaginator, make_request_options
from ....types.vectorize import (
    index_query_params,
    index_create_params,
    index_insert_params,
    index_upsert_params,
    index_get_by_ids_params,
    index_list_vectors_params,
    index_delete_by_ids_params,
)
from ....types.vectorize.create_index import CreateIndex
from ....types.vectorize.index_info_response import IndexInfoResponse
from ....types.vectorize.index_query_response import IndexQueryResponse
from ....types.vectorize.index_delete_response import IndexDeleteResponse
from ....types.vectorize.index_insert_response import IndexInsertResponse
from ....types.vectorize.index_upsert_response import IndexUpsertResponse
from ....types.vectorize.index_list_vectors_response import IndexListVectorsResponse
from ....types.vectorize.index_delete_by_ids_response import IndexDeleteByIDsResponse

__all__ = ["IndexesResource", "AsyncIndexesResource"]


class IndexesResource(SyncAPIResource):
    @cached_property
    def metadata_index(self) -> MetadataIndexResource:
        return MetadataIndexResource(self._client)

    @cached_property
    def with_raw_response(self) -> IndexesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return IndexesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IndexesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return IndexesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        config: index_create_params.Config,
        name: str,
        description: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CreateIndex]:
        """
        Creates and returns a new Vectorize Index.

        Args:
          account_id: Identifier

          config: Specifies the type of configuration to use for the index.

          description: Specifies the description of the index.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/vectorize/v2/indexes", account_id=account_id),
            body=maybe_transform(
                {
                    "config": config,
                    "name": name,
                    "description": description,
                },
                index_create_params.IndexCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CreateIndex]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CreateIndex]], ResultWrapper[CreateIndex]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[CreateIndex]:
        """
        Returns a list of Vectorize Indexes

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/vectorize/v2/indexes", account_id=account_id),
            page=SyncSinglePage[CreateIndex],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CreateIndex,
        )

    def delete(
        self,
        index_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndexDeleteResponse]:
        """
        Deletes the specified Vectorize Index.

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return cast(
            Optional[IndexDeleteResponse],
            self._delete(
                path_template(
                    "/accounts/{account_id}/vectorize/v2/indexes/{index_name}",
                    account_id=account_id,
                    index_name=index_name,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[IndexDeleteResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[IndexDeleteResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def delete_by_ids(
        self,
        index_name: str,
        *,
        account_id: str,
        ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndexDeleteByIDsResponse]:
        """
        Delete a set of vectors from an index by their vector identifiers.

        Args:
          account_id: Identifier

          ids: A list of vector identifiers to delete from the index indicated by the path.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}/delete_by_ids",
                account_id=account_id,
                index_name=index_name,
            ),
            body=maybe_transform({"ids": ids}, index_delete_by_ids_params.IndexDeleteByIDsParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[IndexDeleteByIDsResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndexDeleteByIDsResponse]], ResultWrapper[IndexDeleteByIDsResponse]),
        )

    def get(
        self,
        index_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CreateIndex]:
        """
        Returns the specified Vectorize Index.

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}", account_id=account_id, index_name=index_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CreateIndex]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CreateIndex]], ResultWrapper[CreateIndex]),
        )

    def get_by_ids(
        self,
        index_name: str,
        *,
        account_id: str,
        ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Get a set of vectors from an index by their vector identifiers.

        Args:
          account_id: Identifier

          ids: A list of vector identifiers to retrieve from the index indicated by the path.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}/get_by_ids",
                account_id=account_id,
                index_name=index_name,
            ),
            body=maybe_transform({"ids": ids}, index_get_by_ids_params.IndexGetByIDsParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def info(
        self,
        index_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndexInfoResponse]:
        """
        Get information about a vectorize index.

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}/info",
                account_id=account_id,
                index_name=index_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[IndexInfoResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndexInfoResponse]], ResultWrapper[IndexInfoResponse]),
        )

    def insert(
        self,
        index_name: str,
        *,
        account_id: str,
        body: FileTypes,
        unparsable_behavior: Literal["error", "discard"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndexInsertResponse]:
        """
        Inserts vectors into the specified index and returns a mutation id corresponding
        to the vectors enqueued for insertion.

        Args:
          account_id: Identifier

          body: ndjson file containing vectors to insert.

          unparsable_behavior: Behavior for ndjson parse failures.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}/insert",
                account_id=account_id,
                index_name=index_name,
            ),
            body=maybe_transform(body, index_insert_params.IndexInsertParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"unparsable_behavior": unparsable_behavior}, index_insert_params.IndexInsertParams
                ),
                post_parser=ResultWrapper[Optional[IndexInsertResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndexInsertResponse]], ResultWrapper[IndexInsertResponse]),
        )

    def list_vectors(
        self,
        index_name: str,
        *,
        account_id: str,
        count: int | Omit = omit,
        cursor: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndexListVectorsResponse]:
        """
        Returns a paginated list of vector identifiers from the specified index.

        Args:
          account_id: Identifier

          count: Maximum number of vectors to return

          cursor: Cursor for pagination to get the next page of results

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}/list",
                account_id=account_id,
                index_name=index_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "count": count,
                        "cursor": cursor,
                    },
                    index_list_vectors_params.IndexListVectorsParams,
                ),
                post_parser=ResultWrapper[Optional[IndexListVectorsResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndexListVectorsResponse]], ResultWrapper[IndexListVectorsResponse]),
        )

    def query(
        self,
        index_name: str,
        *,
        account_id: str,
        vector: Iterable[float],
        filter: object | Omit = omit,
        return_metadata: Literal["none", "indexed", "all"] | Omit = omit,
        return_values: bool | Omit = omit,
        top_k: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndexQueryResponse]:
        """
        Finds vectors closest to a given vector in an index.

        Args:
          account_id: Identifier

          vector: The search vector that will be used to find the nearest neighbors.

          filter: A metadata filter expression used to limit nearest neighbor results.

          return_metadata: Whether to return no metadata, indexed metadata or all metadata associated with
              the closest vectors.

          return_values: Whether to return the values associated with the closest vectors.

          top_k: The number of nearest neighbors to find.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}/query",
                account_id=account_id,
                index_name=index_name,
            ),
            body=maybe_transform(
                {
                    "vector": vector,
                    "filter": filter,
                    "return_metadata": return_metadata,
                    "return_values": return_values,
                    "top_k": top_k,
                },
                index_query_params.IndexQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[IndexQueryResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndexQueryResponse]], ResultWrapper[IndexQueryResponse]),
        )

    def upsert(
        self,
        index_name: str,
        *,
        account_id: str,
        body: FileTypes,
        unparsable_behavior: Literal["error", "discard"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndexUpsertResponse]:
        """
        Upserts vectors into the specified index, creating them if they do not exist and
        returns a mutation id corresponding to the vectors enqueued for upsertion.

        Args:
          account_id: Identifier

          body: ndjson file containing vectors to upsert.

          unparsable_behavior: Behavior for ndjson parse failures.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}/upsert",
                account_id=account_id,
                index_name=index_name,
            ),
            body=maybe_transform(body, index_upsert_params.IndexUpsertParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"unparsable_behavior": unparsable_behavior}, index_upsert_params.IndexUpsertParams
                ),
                post_parser=ResultWrapper[Optional[IndexUpsertResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndexUpsertResponse]], ResultWrapper[IndexUpsertResponse]),
        )


class AsyncIndexesResource(AsyncAPIResource):
    @cached_property
    def metadata_index(self) -> AsyncMetadataIndexResource:
        return AsyncMetadataIndexResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncIndexesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncIndexesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIndexesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncIndexesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        config: index_create_params.Config,
        name: str,
        description: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CreateIndex]:
        """
        Creates and returns a new Vectorize Index.

        Args:
          account_id: Identifier

          config: Specifies the type of configuration to use for the index.

          description: Specifies the description of the index.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/vectorize/v2/indexes", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "config": config,
                    "name": name,
                    "description": description,
                },
                index_create_params.IndexCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CreateIndex]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CreateIndex]], ResultWrapper[CreateIndex]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CreateIndex, AsyncSinglePage[CreateIndex]]:
        """
        Returns a list of Vectorize Indexes

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/vectorize/v2/indexes", account_id=account_id),
            page=AsyncSinglePage[CreateIndex],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=CreateIndex,
        )

    async def delete(
        self,
        index_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndexDeleteResponse]:
        """
        Deletes the specified Vectorize Index.

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return cast(
            Optional[IndexDeleteResponse],
            await self._delete(
                path_template(
                    "/accounts/{account_id}/vectorize/v2/indexes/{index_name}",
                    account_id=account_id,
                    index_name=index_name,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[IndexDeleteResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[IndexDeleteResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def delete_by_ids(
        self,
        index_name: str,
        *,
        account_id: str,
        ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndexDeleteByIDsResponse]:
        """
        Delete a set of vectors from an index by their vector identifiers.

        Args:
          account_id: Identifier

          ids: A list of vector identifiers to delete from the index indicated by the path.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}/delete_by_ids",
                account_id=account_id,
                index_name=index_name,
            ),
            body=await async_maybe_transform({"ids": ids}, index_delete_by_ids_params.IndexDeleteByIDsParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[IndexDeleteByIDsResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndexDeleteByIDsResponse]], ResultWrapper[IndexDeleteByIDsResponse]),
        )

    async def get(
        self,
        index_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[CreateIndex]:
        """
        Returns the specified Vectorize Index.

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}", account_id=account_id, index_name=index_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[CreateIndex]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[CreateIndex]], ResultWrapper[CreateIndex]),
        )

    async def get_by_ids(
        self,
        index_name: str,
        *,
        account_id: str,
        ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Get a set of vectors from an index by their vector identifiers.

        Args:
          account_id: Identifier

          ids: A list of vector identifiers to retrieve from the index indicated by the path.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}/get_by_ids",
                account_id=account_id,
                index_name=index_name,
            ),
            body=await async_maybe_transform({"ids": ids}, index_get_by_ids_params.IndexGetByIDsParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def info(
        self,
        index_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndexInfoResponse]:
        """
        Get information about a vectorize index.

        Args:
          account_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}/info",
                account_id=account_id,
                index_name=index_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[IndexInfoResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndexInfoResponse]], ResultWrapper[IndexInfoResponse]),
        )

    async def insert(
        self,
        index_name: str,
        *,
        account_id: str,
        body: FileTypes,
        unparsable_behavior: Literal["error", "discard"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndexInsertResponse]:
        """
        Inserts vectors into the specified index and returns a mutation id corresponding
        to the vectors enqueued for insertion.

        Args:
          account_id: Identifier

          body: ndjson file containing vectors to insert.

          unparsable_behavior: Behavior for ndjson parse failures.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}/insert",
                account_id=account_id,
                index_name=index_name,
            ),
            body=await async_maybe_transform(body, index_insert_params.IndexInsertParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"unparsable_behavior": unparsable_behavior}, index_insert_params.IndexInsertParams
                ),
                post_parser=ResultWrapper[Optional[IndexInsertResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndexInsertResponse]], ResultWrapper[IndexInsertResponse]),
        )

    async def list_vectors(
        self,
        index_name: str,
        *,
        account_id: str,
        count: int | Omit = omit,
        cursor: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndexListVectorsResponse]:
        """
        Returns a paginated list of vector identifiers from the specified index.

        Args:
          account_id: Identifier

          count: Maximum number of vectors to return

          cursor: Cursor for pagination to get the next page of results

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}/list",
                account_id=account_id,
                index_name=index_name,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "count": count,
                        "cursor": cursor,
                    },
                    index_list_vectors_params.IndexListVectorsParams,
                ),
                post_parser=ResultWrapper[Optional[IndexListVectorsResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndexListVectorsResponse]], ResultWrapper[IndexListVectorsResponse]),
        )

    async def query(
        self,
        index_name: str,
        *,
        account_id: str,
        vector: Iterable[float],
        filter: object | Omit = omit,
        return_metadata: Literal["none", "indexed", "all"] | Omit = omit,
        return_values: bool | Omit = omit,
        top_k: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndexQueryResponse]:
        """
        Finds vectors closest to a given vector in an index.

        Args:
          account_id: Identifier

          vector: The search vector that will be used to find the nearest neighbors.

          filter: A metadata filter expression used to limit nearest neighbor results.

          return_metadata: Whether to return no metadata, indexed metadata or all metadata associated with
              the closest vectors.

          return_values: Whether to return the values associated with the closest vectors.

          top_k: The number of nearest neighbors to find.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}/query",
                account_id=account_id,
                index_name=index_name,
            ),
            body=await async_maybe_transform(
                {
                    "vector": vector,
                    "filter": filter,
                    "return_metadata": return_metadata,
                    "return_values": return_values,
                    "top_k": top_k,
                },
                index_query_params.IndexQueryParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[IndexQueryResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndexQueryResponse]], ResultWrapper[IndexQueryResponse]),
        )

    async def upsert(
        self,
        index_name: str,
        *,
        account_id: str,
        body: FileTypes,
        unparsable_behavior: Literal["error", "discard"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[IndexUpsertResponse]:
        """
        Upserts vectors into the specified index, creating them if they do not exist and
        returns a mutation id corresponding to the vectors enqueued for upsertion.

        Args:
          account_id: Identifier

          body: ndjson file containing vectors to upsert.

          unparsable_behavior: Behavior for ndjson parse failures.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not index_name:
            raise ValueError(f"Expected a non-empty value for `index_name` but received {index_name!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/vectorize/v2/indexes/{index_name}/upsert",
                account_id=account_id,
                index_name=index_name,
            ),
            body=await async_maybe_transform(body, index_upsert_params.IndexUpsertParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"unparsable_behavior": unparsable_behavior}, index_upsert_params.IndexUpsertParams
                ),
                post_parser=ResultWrapper[Optional[IndexUpsertResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[IndexUpsertResponse]], ResultWrapper[IndexUpsertResponse]),
        )


class IndexesResourceWithRawResponse:
    def __init__(self, indexes: IndexesResource) -> None:
        self._indexes = indexes

        self.create = to_raw_response_wrapper(
            indexes.create,
        )
        self.list = to_raw_response_wrapper(
            indexes.list,
        )
        self.delete = to_raw_response_wrapper(
            indexes.delete,
        )
        self.delete_by_ids = to_raw_response_wrapper(
            indexes.delete_by_ids,
        )
        self.get = to_raw_response_wrapper(
            indexes.get,
        )
        self.get_by_ids = to_raw_response_wrapper(
            indexes.get_by_ids,
        )
        self.info = to_raw_response_wrapper(
            indexes.info,
        )
        self.insert = to_raw_response_wrapper(
            indexes.insert,
        )
        self.list_vectors = to_raw_response_wrapper(
            indexes.list_vectors,
        )
        self.query = to_raw_response_wrapper(
            indexes.query,
        )
        self.upsert = to_raw_response_wrapper(
            indexes.upsert,
        )

    @cached_property
    def metadata_index(self) -> MetadataIndexResourceWithRawResponse:
        return MetadataIndexResourceWithRawResponse(self._indexes.metadata_index)


class AsyncIndexesResourceWithRawResponse:
    def __init__(self, indexes: AsyncIndexesResource) -> None:
        self._indexes = indexes

        self.create = async_to_raw_response_wrapper(
            indexes.create,
        )
        self.list = async_to_raw_response_wrapper(
            indexes.list,
        )
        self.delete = async_to_raw_response_wrapper(
            indexes.delete,
        )
        self.delete_by_ids = async_to_raw_response_wrapper(
            indexes.delete_by_ids,
        )
        self.get = async_to_raw_response_wrapper(
            indexes.get,
        )
        self.get_by_ids = async_to_raw_response_wrapper(
            indexes.get_by_ids,
        )
        self.info = async_to_raw_response_wrapper(
            indexes.info,
        )
        self.insert = async_to_raw_response_wrapper(
            indexes.insert,
        )
        self.list_vectors = async_to_raw_response_wrapper(
            indexes.list_vectors,
        )
        self.query = async_to_raw_response_wrapper(
            indexes.query,
        )
        self.upsert = async_to_raw_response_wrapper(
            indexes.upsert,
        )

    @cached_property
    def metadata_index(self) -> AsyncMetadataIndexResourceWithRawResponse:
        return AsyncMetadataIndexResourceWithRawResponse(self._indexes.metadata_index)


class IndexesResourceWithStreamingResponse:
    def __init__(self, indexes: IndexesResource) -> None:
        self._indexes = indexes

        self.create = to_streamed_response_wrapper(
            indexes.create,
        )
        self.list = to_streamed_response_wrapper(
            indexes.list,
        )
        self.delete = to_streamed_response_wrapper(
            indexes.delete,
        )
        self.delete_by_ids = to_streamed_response_wrapper(
            indexes.delete_by_ids,
        )
        self.get = to_streamed_response_wrapper(
            indexes.get,
        )
        self.get_by_ids = to_streamed_response_wrapper(
            indexes.get_by_ids,
        )
        self.info = to_streamed_response_wrapper(
            indexes.info,
        )
        self.insert = to_streamed_response_wrapper(
            indexes.insert,
        )
        self.list_vectors = to_streamed_response_wrapper(
            indexes.list_vectors,
        )
        self.query = to_streamed_response_wrapper(
            indexes.query,
        )
        self.upsert = to_streamed_response_wrapper(
            indexes.upsert,
        )

    @cached_property
    def metadata_index(self) -> MetadataIndexResourceWithStreamingResponse:
        return MetadataIndexResourceWithStreamingResponse(self._indexes.metadata_index)


class AsyncIndexesResourceWithStreamingResponse:
    def __init__(self, indexes: AsyncIndexesResource) -> None:
        self._indexes = indexes

        self.create = async_to_streamed_response_wrapper(
            indexes.create,
        )
        self.list = async_to_streamed_response_wrapper(
            indexes.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            indexes.delete,
        )
        self.delete_by_ids = async_to_streamed_response_wrapper(
            indexes.delete_by_ids,
        )
        self.get = async_to_streamed_response_wrapper(
            indexes.get,
        )
        self.get_by_ids = async_to_streamed_response_wrapper(
            indexes.get_by_ids,
        )
        self.info = async_to_streamed_response_wrapper(
            indexes.info,
        )
        self.insert = async_to_streamed_response_wrapper(
            indexes.insert,
        )
        self.list_vectors = async_to_streamed_response_wrapper(
            indexes.list_vectors,
        )
        self.query = async_to_streamed_response_wrapper(
            indexes.query,
        )
        self.upsert = async_to_streamed_response_wrapper(
            indexes.upsert,
        )

    @cached_property
    def metadata_index(self) -> AsyncMetadataIndexResourceWithStreamingResponse:
        return AsyncMetadataIndexResourceWithStreamingResponse(self._indexes.metadata_index)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["EveryoneRuleParam", "Everyone"]


class Everyone(TypedDict, total=False):
    """An empty object which matches on all users."""

    pass


class EveryoneRuleParam(TypedDict, total=False):
    """Matches everyone."""

    everyone: Required[Everyone]
    """An empty object which matches on all users."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["AgentReadinessSummaryResponse", "Meta", "MetaDomainCategory", "MetaUnit"]


class MetaDomainCategory(BaseModel):
    name: str
    """Sub-category name."""

    value: int
    """Number of successfully scanned domains in this sub-category."""


class MetaUnit(BaseModel):
    name: str

    value: str


class Meta(BaseModel):
    date: str
    """Date of the returned scan (YYYY-MM-DD).

    May differ from the requested date if no scan exists for that exact date.
    """

    domain_categories: List[MetaDomainCategory] = FieldInfo(alias="domainCategories")
    """Available domain sub-categories with their scan counts.

    Use as filter options for the domainCategory parameter.
    """

    last_updated: datetime = FieldInfo(alias="lastUpdated")
    """Timestamp of the last dataset update."""

    normalization: Literal[
        "PERCENTAGE",
        "MIN0_MAX",
        "MIN_MAX",
        "RAW_VALUES",
        "PERCENTAGE_CHANGE",
        "ROLLING_AVERAGE",
        "OVERLAPPED_PERCENTAGE",
        "RATIO",
    ]
    """Normalization method applied to the results.

    Refer to
    [Normalization methods](https://developers.cloudflare.com/radar/concepts/normalization/).
    """

    successful_domains: int = FieldInfo(alias="successfulDomains")
    """Domains successfully scanned (excludes errors)."""

    total_domains: int = FieldInfo(alias="totalDomains")
    """Total domains attempted in the scan."""

    units: List[MetaUnit]
    """Measurement units for the results."""


class AgentReadinessSummaryResponse(BaseModel):
    meta: Meta

    summary_0: Dict[str, str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel

__all__ = ["CountryRule", "Geo"]


class Geo(BaseModel):
    country_code: str
    """The country code that should be matched."""


class CountryRule(BaseModel):
    """Matches a specific country"""

    geo: Geo



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Iterable, cast

import httpx

from .full import (
    FullResource,
    AsyncFullResource,
    FullResourceWithRawResponse,
    AsyncFullResourceWithRawResponse,
    FullResourceWithStreamingResponse,
    AsyncFullResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.magic_network_monitoring import config_edit_params, config_create_params, config_update_params
from ....types.magic_network_monitoring.configuration import Configuration

__all__ = ["ConfigsResource", "AsyncConfigsResource"]


class ConfigsResource(SyncAPIResource):
    @cached_property
    def full(self) -> FullResource:
        return FullResource(self._client)

    @cached_property
    def with_raw_response(self) -> ConfigsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ConfigsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConfigsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ConfigsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        default_sampling: float,
        name: str,
        router_ips: SequenceNotStr[str] | Omit = omit,
        warp_devices: Iterable[config_create_params.WARPDevice] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Configuration:
        """
        Create a new network monitoring configuration.

        Args:
          default_sampling: Fallback sampling rate of flow messages being sent in packets per second. This
              should match the packet sampling rate configured on the router.

          name: The account name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/mnm/config", account_id=account_id),
            body=maybe_transform(
                {
                    "default_sampling": default_sampling,
                    "name": name,
                    "router_ips": router_ips,
                    "warp_devices": warp_devices,
                },
                config_create_params.ConfigCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Configuration]._unwrapper,
            ),
            cast_to=cast(Type[Configuration], ResultWrapper[Configuration]),
        )

    def update(
        self,
        *,
        account_id: str,
        default_sampling: float,
        name: str,
        router_ips: SequenceNotStr[str] | Omit = omit,
        warp_devices: Iterable[config_update_params.WARPDevice] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Configuration:
        """
        Update an existing network monitoring configuration, requires the entire
        configuration to be updated at once.

        Args:
          default_sampling: Fallback sampling rate of flow messages being sent in packets per second. This
              should match the packet sampling rate configured on the router.

          name: The account name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/mnm/config", account_id=account_id),
            body=maybe_transform(
                {
                    "default_sampling": default_sampling,
                    "name": name,
                    "router_ips": router_ips,
                    "warp_devices": warp_devices,
                },
                config_update_params.ConfigUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Configuration]._unwrapper,
            ),
            cast_to=cast(Type[Configuration], ResultWrapper[Configuration]),
        )

    def delete(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Configuration:
        """
        Delete an existing network monitoring configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/mnm/config", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Configuration]._unwrapper,
            ),
            cast_to=cast(Type[Configuration], ResultWrapper[Configuration]),
        )

    def edit(
        self,
        *,
        account_id: str,
        default_sampling: float | Omit = omit,
        name: str | Omit = omit,
        router_ips: SequenceNotStr[str] | Omit = omit,
        warp_devices: Iterable[config_edit_params.WARPDevice] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Configuration:
        """
        Update fields in an existing network monitoring configuration.

        Args:
          default_sampling: Fallback sampling rate of flow messages being sent in packets per second. This
              should match the packet sampling rate configured on the router.

          name: The account name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._patch(
            path_template("/accounts/{account_id}/mnm/config", account_id=account_id),
            body=maybe_transform(
                {
                    "default_sampling": default_sampling,
                    "name": name,
                    "router_ips": router_ips,
                    "warp_devices": warp_devices,
                },
                config_edit_params.ConfigEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Configuration]._unwrapper,
            ),
            cast_to=cast(Type[Configuration], ResultWrapper[Configuration]),
        )

    def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Configuration:
        """
        Lists default sampling, router IPs and warp devices for account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/mnm/config", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Configuration]._unwrapper,
            ),
            cast_to=cast(Type[Configuration], ResultWrapper[Configuration]),
        )


class AsyncConfigsResource(AsyncAPIResource):
    @cached_property
    def full(self) -> AsyncFullResource:
        return AsyncFullResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncConfigsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConfigsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConfigsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncConfigsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        default_sampling: float,
        name: str,
        router_ips: SequenceNotStr[str] | Omit = omit,
        warp_devices: Iterable[config_create_params.WARPDevice] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Configuration:
        """
        Create a new network monitoring configuration.

        Args:
          default_sampling: Fallback sampling rate of flow messages being sent in packets per second. This
              should match the packet sampling rate configured on the router.

          name: The account name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/mnm/config", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "default_sampling": default_sampling,
                    "name": name,
                    "router_ips": router_ips,
                    "warp_devices": warp_devices,
                },
                config_create_params.ConfigCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Configuration]._unwrapper,
            ),
            cast_to=cast(Type[Configuration], ResultWrapper[Configuration]),
        )

    async def update(
        self,
        *,
        account_id: str,
        default_sampling: float,
        name: str,
        router_ips: SequenceNotStr[str] | Omit = omit,
        warp_devices: Iterable[config_update_params.WARPDevice] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Configuration:
        """
        Update an existing network monitoring configuration, requires the entire
        configuration to be updated at once.

        Args:
          default_sampling: Fallback sampling rate of flow messages being sent in packets per second. This
              should match the packet sampling rate configured on the router.

          name: The account name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/mnm/config", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "default_sampling": default_sampling,
                    "name": name,
                    "router_ips": router_ips,
                    "warp_devices": warp_devices,
                },
                config_update_params.ConfigUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Configuration]._unwrapper,
            ),
            cast_to=cast(Type[Configuration], ResultWrapper[Configuration]),
        )

    async def delete(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Configuration:
        """
        Delete an existing network monitoring configuration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/mnm/config", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Configuration]._unwrapper,
            ),
            cast_to=cast(Type[Configuration], ResultWrapper[Configuration]),
        )

    async def edit(
        self,
        *,
        account_id: str,
        default_sampling: float | Omit = omit,
        name: str | Omit = omit,
        router_ips: SequenceNotStr[str] | Omit = omit,
        warp_devices: Iterable[config_edit_params.WARPDevice] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Configuration:
        """
        Update fields in an existing network monitoring configuration.

        Args:
          default_sampling: Fallback sampling rate of flow messages being sent in packets per second. This
              should match the packet sampling rate configured on the router.

          name: The account name.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._patch(
            path_template("/accounts/{account_id}/mnm/config", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "default_sampling": default_sampling,
                    "name": name,
                    "router_ips": router_ips,
                    "warp_devices": warp_devices,
                },
                config_edit_params.ConfigEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Configuration]._unwrapper,
            ),
            cast_to=cast(Type[Configuration], ResultWrapper[Configuration]),
        )

    async def get(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Configuration:
        """
        Lists default sampling, router IPs and warp devices for account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/mnm/config", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Configuration]._unwrapper,
            ),
            cast_to=cast(Type[Configuration], ResultWrapper[Configuration]),
        )


class ConfigsResourceWithRawResponse:
    def __init__(self, configs: ConfigsResource) -> None:
        self._configs = configs

        self.create = to_raw_response_wrapper(
            configs.create,
        )
        self.update = to_raw_response_wrapper(
            configs.update,
        )
        self.delete = to_raw_response_wrapper(
            configs.delete,
        )
        self.edit = to_raw_response_wrapper(
            configs.edit,
        )
        self.get = to_raw_response_wrapper(
            configs.get,
        )

    @cached_property
    def full(self) -> FullResourceWithRawResponse:
        return FullResourceWithRawResponse(self._configs.full)


class AsyncConfigsResourceWithRawResponse:
    def __init__(self, configs: AsyncConfigsResource) -> None:
        self._configs = configs

        self.create = async_to_raw_response_wrapper(
            configs.create,
        )
        self.update = async_to_raw_response_wrapper(
            configs.update,
        )
        self.delete = async_to_raw_response_wrapper(
            configs.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            configs.edit,
        )
        self.get = async_to_raw_response_wrapper(
            configs.get,
        )

    @cached_property
    def full(self) -> AsyncFullResourceWithRawResponse:
        return AsyncFullResourceWithRawResponse(self._configs.full)


class ConfigsResourceWithStreamingResponse:
    def __init__(self, configs: ConfigsResource) -> None:
        self._configs = configs

        self.create = to_streamed_response_wrapper(
            configs.create,
        )
        self.update = to_streamed_response_wrapper(
            configs.update,
        )
        self.delete = to_streamed_response_wrapper(
            configs.delete,
        )
        self.edit = to_streamed_response_wrapper(
            configs.edit,
        )
        self.get = to_streamed_response_wrapper(
            configs.get,
        )

    @cached_property
    def full(self) -> FullResourceWithStreamingResponse:
        return FullResourceWithStreamingResponse(self._configs.full)


class AsyncConfigsResourceWithStreamingResponse:
    def __init__(self, configs: AsyncConfigsResource) -> None:
        self._configs = configs

        self.create = async_to_streamed_response_wrapper(
            configs.create,
        )
        self.update = async_to_streamed_response_wrapper(
            configs.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            configs.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            configs.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            configs.get,
        )

    @cached_property
    def full(self) -> AsyncFullResourceWithStreamingResponse:
        return AsyncFullResourceWithStreamingResponse(self._configs.full)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .nat_param import NatParam
from .routed_subnet_param import RoutedSubnetParam
from .lan_static_addressing_param import LANStaticAddressingParam

__all__ = ["LANUpdateParams"]


class LANUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    site_id: Required[str]
    """Identifier"""

    bond_id: int

    is_breakout: bool
    """mark true to use this LAN for source-based breakout traffic"""

    is_prioritized: bool
    """mark true to use this LAN for source-based prioritized traffic"""

    name: str

    nat: NatParam

    physport: int

    routed_subnets: Iterable[RoutedSubnetParam]

    static_addressing: LANStaticAddressingParam
    """
    If the site is not configured in high availability mode, this configuration is
    optional (if omitted, use DHCP). However, if in high availability mode,
    static_address is required along with secondary and virtual address.
    """

    vlan_tag: int
    """VLAN ID. Use zero for untagged."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .hostnames import (
    HostnamesResource,
    AsyncHostnamesResource,
    HostnamesResourceWithRawResponse,
    AsyncHostnamesResourceWithRawResponse,
    HostnamesResourceWithStreamingResponse,
    AsyncHostnamesResourceWithStreamingResponse,
)

__all__ = [
    "SettingsResource",
    "AsyncSettingsResource",
    "SettingsResourceWithRawResponse",
    "AsyncSettingsResourceWithRawResponse",
    "SettingsResourceWithStreamingResponse",
    "AsyncSettingsResourceWithStreamingResponse",
    "HostnamesResource",
    "AsyncHostnamesResource",
    "HostnamesResourceWithRawResponse",
    "AsyncHostnamesResourceWithRawResponse",
    "HostnamesResourceWithStreamingResponse",
    "AsyncHostnamesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from .dns import (
    DNSResource,
    AsyncDNSResource,
    DNSResourceWithRawResponse,
    AsyncDNSResourceWithRawResponse,
    DNSResourceWithStreamingResponse,
    AsyncDNSResourceWithStreamingResponse,
)
from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.email_sending import subdomain_create_params
from ....types.email_sending.subdomain_get_response import SubdomainGetResponse
from ....types.email_sending.subdomain_list_response import SubdomainListResponse
from ....types.email_sending.subdomain_create_response import SubdomainCreateResponse
from ....types.email_sending.subdomain_delete_response import SubdomainDeleteResponse

__all__ = ["SubdomainsResource", "AsyncSubdomainsResource"]


class SubdomainsResource(SyncAPIResource):
    @cached_property
    def dns(self) -> DNSResource:
        return DNSResource(self._client)

    @cached_property
    def with_raw_response(self) -> SubdomainsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SubdomainsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SubdomainsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SubdomainsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        zone_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SubdomainCreateResponse]:
        """
        Creates a new sending subdomain or re-enables sending on an existing subdomain
        that had it disabled. If zone-level Email Sending has not been enabled yet, the
        zone flag is automatically set when the entitlement is present.

        Args:
          zone_id: Identifier.

          name: The subdomain name. Must be within the zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._post(
            path_template("/zones/{zone_id}/email/sending/subdomains", zone_id=zone_id),
            body=maybe_transform({"name": name}, subdomain_create_params.SubdomainCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SubdomainCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SubdomainCreateResponse]], ResultWrapper[SubdomainCreateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[SubdomainListResponse]:
        """
        Lists all sending-enabled subdomains for the zone.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/email/sending/subdomains", zone_id=zone_id),
            page=SyncSinglePage[SubdomainListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=SubdomainListResponse,
        )

    def delete(
        self,
        subdomain_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubdomainDeleteResponse:
        """Disables sending on a subdomain and removes its DNS records.

        If routing is still
        active on the subdomain, only sending is disabled.

        Args:
          zone_id: Identifier.

          subdomain_id: Sending subdomain identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not subdomain_id:
            raise ValueError(f"Expected a non-empty value for `subdomain_id` but received {subdomain_id!r}")
        return self._delete(
            path_template(
                "/zones/{zone_id}/email/sending/subdomains/{subdomain_id}", zone_id=zone_id, subdomain_id=subdomain_id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubdomainDeleteResponse,
        )

    def get(
        self,
        subdomain_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SubdomainGetResponse]:
        """
        Gets information for a specific sending subdomain.

        Args:
          zone_id: Identifier.

          subdomain_id: Sending subdomain identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not subdomain_id:
            raise ValueError(f"Expected a non-empty value for `subdomain_id` but received {subdomain_id!r}")
        return self._get(
            path_template(
                "/zones/{zone_id}/email/sending/subdomains/{subdomain_id}", zone_id=zone_id, subdomain_id=subdomain_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SubdomainGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SubdomainGetResponse]], ResultWrapper[SubdomainGetResponse]),
        )


class AsyncSubdomainsResource(AsyncAPIResource):
    @cached_property
    def dns(self) -> AsyncDNSResource:
        return AsyncDNSResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSubdomainsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSubdomainsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSubdomainsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSubdomainsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        zone_id: str,
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SubdomainCreateResponse]:
        """
        Creates a new sending subdomain or re-enables sending on an existing subdomain
        that had it disabled. If zone-level Email Sending has not been enabled yet, the
        zone flag is automatically set when the entitlement is present.

        Args:
          zone_id: Identifier.

          name: The subdomain name. Must be within the zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._post(
            path_template("/zones/{zone_id}/email/sending/subdomains", zone_id=zone_id),
            body=await async_maybe_transform({"name": name}, subdomain_create_params.SubdomainCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SubdomainCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SubdomainCreateResponse]], ResultWrapper[SubdomainCreateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[SubdomainListResponse, AsyncSinglePage[SubdomainListResponse]]:
        """
        Lists all sending-enabled subdomains for the zone.

        Args:
          zone_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/email/sending/subdomains", zone_id=zone_id),
            page=AsyncSinglePage[SubdomainListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=SubdomainListResponse,
        )

    async def delete(
        self,
        subdomain_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SubdomainDeleteResponse:
        """Disables sending on a subdomain and removes its DNS records.

        If routing is still
        active on the subdomain, only sending is disabled.

        Args:
          zone_id: Identifier.

          subdomain_id: Sending subdomain identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not subdomain_id:
            raise ValueError(f"Expected a non-empty value for `subdomain_id` but received {subdomain_id!r}")
        return await self._delete(
            path_template(
                "/zones/{zone_id}/email/sending/subdomains/{subdomain_id}", zone_id=zone_id, subdomain_id=subdomain_id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SubdomainDeleteResponse,
        )

    async def get(
        self,
        subdomain_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[SubdomainGetResponse]:
        """
        Gets information for a specific sending subdomain.

        Args:
          zone_id: Identifier.

          subdomain_id: Sending subdomain identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not subdomain_id:
            raise ValueError(f"Expected a non-empty value for `subdomain_id` but received {subdomain_id!r}")
        return await self._get(
            path_template(
                "/zones/{zone_id}/email/sending/subdomains/{subdomain_id}", zone_id=zone_id, subdomain_id=subdomain_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[SubdomainGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[SubdomainGetResponse]], ResultWrapper[SubdomainGetResponse]),
        )


class SubdomainsResourceWithRawResponse:
    def __init__(self, subdomains: SubdomainsResource) -> None:
        self._subdomains = subdomains

        self.create = to_raw_response_wrapper(
            subdomains.create,
        )
        self.list = to_raw_response_wrapper(
            subdomains.list,
        )
        self.delete = to_raw_response_wrapper(
            subdomains.delete,
        )
        self.get = to_raw_response_wrapper(
            subdomains.get,
        )

    @cached_property
    def dns(self) -> DNSResourceWithRawResponse:
        return DNSResourceWithRawResponse(self._subdomains.dns)


class AsyncSubdomainsResourceWithRawResponse:
    def __init__(self, subdomains: AsyncSubdomainsResource) -> None:
        self._subdomains = subdomains

        self.create = async_to_raw_response_wrapper(
            subdomains.create,
        )
        self.list = async_to_raw_response_wrapper(
            subdomains.list,
        )
        self.delete = async_to_raw_response_wrapper(
            subdomains.delete,
        )
        self.get = async_to_raw_response_wrapper(
            subdomains.get,
        )

    @cached_property
    def dns(self) -> AsyncDNSResourceWithRawResponse:
        return AsyncDNSResourceWithRawResponse(self._subdomains.dns)


class SubdomainsResourceWithStreamingResponse:
    def __init__(self, subdomains: SubdomainsResource) -> None:
        self._subdomains = subdomains

        self.create = to_streamed_response_wrapper(
            subdomains.create,
        )
        self.list = to_streamed_response_wrapper(
            subdomains.list,
        )
        self.delete = to_streamed_response_wrapper(
            subdomains.delete,
        )
        self.get = to_streamed_response_wrapper(
            subdomains.get,
        )

    @cached_property
    def dns(self) -> DNSResourceWithStreamingResponse:
        return DNSResourceWithStreamingResponse(self._subdomains.dns)


class AsyncSubdomainsResourceWithStreamingResponse:
    def __init__(self, subdomains: AsyncSubdomainsResource) -> None:
        self._subdomains = subdomains

        self.create = async_to_streamed_response_wrapper(
            subdomains.create,
        )
        self.list = async_to_streamed_response_wrapper(
            subdomains.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            subdomains.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            subdomains.get,
        )

    @cached_property
    def dns(self) -> AsyncDNSResourceWithStreamingResponse:
        return AsyncDNSResourceWithStreamingResponse(self._subdomains.dns)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from .advertisements import (
    AdvertisementsResource,
    AsyncAdvertisementsResource,
    AdvertisementsResourceWithRawResponse,
    AsyncAdvertisementsResourceWithRawResponse,
    AdvertisementsResourceWithStreamingResponse,
    AsyncAdvertisementsResourceWithStreamingResponse,
)
from ...._base_client import AsyncPaginator, make_request_options
from ....types.magic_network_monitoring import rule_edit_params, rule_create_params, rule_update_params
from ....types.magic_network_monitoring.magic_network_monitoring_rule import MagicNetworkMonitoringRule

__all__ = ["RulesResource", "AsyncRulesResource"]


class RulesResource(SyncAPIResource):
    @cached_property
    def advertisements(self) -> AdvertisementsResource:
        return AdvertisementsResource(self._client)

    @cached_property
    def with_raw_response(self) -> RulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return RulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return RulesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        automatic_advertisement: Optional[bool],
        name: str,
        prefixes: SequenceNotStr[str],
        type: Literal["threshold", "zscore", "advanced_ddos"],
        bandwidth_threshold: float | Omit = omit,
        duration: Literal["1m", "5m", "10m", "15m", "20m", "30m", "45m", "60m"] | Omit = omit,
        packet_threshold: float | Omit = omit,
        prefix_match: Optional[Literal["exact", "subnet", "supernet"]] | Omit = omit,
        zscore_sensitivity: Optional[Literal["low", "medium", "high"]] | Omit = omit,
        zscore_target: Optional[Literal["bits", "packets"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MagicNetworkMonitoringRule]:
        """Create network monitoring rules for account.

        Currently only supports creating a
        single rule per API request.

        Args:
          automatic_advertisement: Toggle on if you would like Cloudflare to automatically advertise the IP
              Prefixes within the rule via Magic Transit when the rule is triggered. Only
              available for users of Magic Transit.

          name: The name of the rule. Must be unique. Supports characters A-Z, a-z, 0-9,
              underscore (\\__), dash (-), period (.), and tilde (~). You can’t have a space in
              the rule name. Max 256 characters.

          type: MNM rule type.

          bandwidth_threshold: The number of bits per second for the rule. When this value is exceeded for the
              set duration, an alert notification is sent. Minimum of 1 and no maximum.

          duration: The amount of time that the rule threshold must be exceeded to send an alert
              notification. The final value must be equivalent to one of the following 8
              values ["1m","5m","10m","15m","20m","30m","45m","60m"].

          packet_threshold: The number of packets per second for the rule. When this value is exceeded for
              the set duration, an alert notification is sent. Minimum of 1 and no maximum.

          prefix_match: Prefix match type to be applied for a prefix auto advertisement when using an
              advanced_ddos rule.

          zscore_sensitivity: Level of sensitivity set for zscore rules.

          zscore_target: Target of the zscore rule analysis.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/mnm/rules", account_id=account_id),
            body=maybe_transform(
                {
                    "automatic_advertisement": automatic_advertisement,
                    "name": name,
                    "prefixes": prefixes,
                    "type": type,
                    "bandwidth_threshold": bandwidth_threshold,
                    "duration": duration,
                    "packet_threshold": packet_threshold,
                    "prefix_match": prefix_match,
                    "zscore_sensitivity": zscore_sensitivity,
                    "zscore_target": zscore_target,
                },
                rule_create_params.RuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MagicNetworkMonitoringRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MagicNetworkMonitoringRule]], ResultWrapper[MagicNetworkMonitoringRule]),
        )

    def update(
        self,
        *,
        account_id: str,
        automatic_advertisement: Optional[bool],
        name: str,
        prefixes: SequenceNotStr[str],
        type: Literal["threshold", "zscore", "advanced_ddos"],
        bandwidth_threshold: float | Omit = omit,
        duration: Literal["1m", "5m", "10m", "15m", "20m", "30m", "45m", "60m"] | Omit = omit,
        packet_threshold: float | Omit = omit,
        prefix_match: Optional[Literal["exact", "subnet", "supernet"]] | Omit = omit,
        zscore_sensitivity: Optional[Literal["low", "medium", "high"]] | Omit = omit,
        zscore_target: Optional[Literal["bits", "packets"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MagicNetworkMonitoringRule]:
        """
        Update network monitoring rules for account.

        Args:
          automatic_advertisement: Toggle on if you would like Cloudflare to automatically advertise the IP
              Prefixes within the rule via Magic Transit when the rule is triggered. Only
              available for users of Magic Transit.

          name: The name of the rule. Must be unique. Supports characters A-Z, a-z, 0-9,
              underscore (\\__), dash (-), period (.), and tilde (~). You can’t have a space in
              the rule name. Max 256 characters.

          type: MNM rule type.

          bandwidth_threshold: The number of bits per second for the rule. When this value is exceeded for the
              set duration, an alert notification is sent. Minimum of 1 and no maximum.

          duration: The amount of time that the rule threshold must be exceeded to send an alert
              notification. The final value must be equivalent to one of the following 8
              values ["1m","5m","10m","15m","20m","30m","45m","60m"].

          packet_threshold: The number of packets per second for the rule. When this value is exceeded for
              the set duration, an alert notification is sent. Minimum of 1 and no maximum.

          prefix_match: Prefix match type to be applied for a prefix auto advertisement when using an
              advanced_ddos rule.

          zscore_sensitivity: Level of sensitivity set for zscore rules.

          zscore_target: Target of the zscore rule analysis.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/mnm/rules", account_id=account_id),
            body=maybe_transform(
                {
                    "automatic_advertisement": automatic_advertisement,
                    "name": name,
                    "prefixes": prefixes,
                    "type": type,
                    "bandwidth_threshold": bandwidth_threshold,
                    "duration": duration,
                    "packet_threshold": packet_threshold,
                    "prefix_match": prefix_match,
                    "zscore_sensitivity": zscore_sensitivity,
                    "zscore_target": zscore_target,
                },
                rule_update_params.RuleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MagicNetworkMonitoringRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MagicNetworkMonitoringRule]], ResultWrapper[MagicNetworkMonitoringRule]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[Optional[MagicNetworkMonitoringRule]]:
        """
        Lists network monitoring rules for account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/mnm/rules", account_id=account_id),
            page=SyncSinglePage[Optional[MagicNetworkMonitoringRule]],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=MagicNetworkMonitoringRule,
        )

    def delete(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MagicNetworkMonitoringRule]:
        """
        Delete a network monitoring rule for account.

        Args:
          rule_id: The id of the rule. Must be unique.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._delete(
            path_template("/accounts/{account_id}/mnm/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MagicNetworkMonitoringRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MagicNetworkMonitoringRule]], ResultWrapper[MagicNetworkMonitoringRule]),
        )

    def edit(
        self,
        rule_id: str,
        *,
        account_id: str,
        automatic_advertisement: Optional[bool],
        name: str,
        prefixes: SequenceNotStr[str],
        type: Literal["threshold", "zscore", "advanced_ddos"],
        bandwidth_threshold: float | Omit = omit,
        duration: Literal["1m", "5m", "10m", "15m", "20m", "30m", "45m", "60m"] | Omit = omit,
        packet_threshold: float | Omit = omit,
        prefix_match: Optional[Literal["exact", "subnet", "supernet"]] | Omit = omit,
        zscore_sensitivity: Optional[Literal["low", "medium", "high"]] | Omit = omit,
        zscore_target: Optional[Literal["bits", "packets"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MagicNetworkMonitoringRule]:
        """
        Update a network monitoring rule for account.

        Args:
          rule_id: The id of the rule. Must be unique.

          automatic_advertisement: Toggle on if you would like Cloudflare to automatically advertise the IP
              Prefixes within the rule via Magic Transit when the rule is triggered. Only
              available for users of Magic Transit.

          name: The name of the rule. Must be unique. Supports characters A-Z, a-z, 0-9,
              underscore (\\__), dash (-), period (.), and tilde (~). You can’t have a space in
              the rule name. Max 256 characters.

          type: MNM rule type.

          bandwidth_threshold: The number of bits per second for the rule. When this value is exceeded for the
              set duration, an alert notification is sent. Minimum of 1 and no maximum.

          duration: The amount of time that the rule threshold must be exceeded to send an alert
              notification. The final value must be equivalent to one of the following 8
              values ["1m","5m","10m","15m","20m","30m","45m","60m"].

          packet_threshold: The number of packets per second for the rule. When this value is exceeded for
              the set duration, an alert notification is sent. Minimum of 1 and no maximum.

          prefix_match: Prefix match type to be applied for a prefix auto advertisement when using an
              advanced_ddos rule.

          zscore_sensitivity: Level of sensitivity set for zscore rules.

          zscore_target: Target of the zscore rule analysis.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._patch(
            path_template("/accounts/{account_id}/mnm/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            body=maybe_transform(
                {
                    "automatic_advertisement": automatic_advertisement,
                    "name": name,
                    "prefixes": prefixes,
                    "type": type,
                    "bandwidth_threshold": bandwidth_threshold,
                    "duration": duration,
                    "packet_threshold": packet_threshold,
                    "prefix_match": prefix_match,
                    "zscore_sensitivity": zscore_sensitivity,
                    "zscore_target": zscore_target,
                },
                rule_edit_params.RuleEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MagicNetworkMonitoringRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MagicNetworkMonitoringRule]], ResultWrapper[MagicNetworkMonitoringRule]),
        )

    def get(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MagicNetworkMonitoringRule]:
        """
        List a single network monitoring rule for account.

        Args:
          rule_id: The id of the rule. Must be unique.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/mnm/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MagicNetworkMonitoringRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MagicNetworkMonitoringRule]], ResultWrapper[MagicNetworkMonitoringRule]),
        )


class AsyncRulesResource(AsyncAPIResource):
    @cached_property
    def advertisements(self) -> AsyncAdvertisementsResource:
        return AsyncAdvertisementsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncRulesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        automatic_advertisement: Optional[bool],
        name: str,
        prefixes: SequenceNotStr[str],
        type: Literal["threshold", "zscore", "advanced_ddos"],
        bandwidth_threshold: float | Omit = omit,
        duration: Literal["1m", "5m", "10m", "15m", "20m", "30m", "45m", "60m"] | Omit = omit,
        packet_threshold: float | Omit = omit,
        prefix_match: Optional[Literal["exact", "subnet", "supernet"]] | Omit = omit,
        zscore_sensitivity: Optional[Literal["low", "medium", "high"]] | Omit = omit,
        zscore_target: Optional[Literal["bits", "packets"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MagicNetworkMonitoringRule]:
        """Create network monitoring rules for account.

        Currently only supports creating a
        single rule per API request.

        Args:
          automatic_advertisement: Toggle on if you would like Cloudflare to automatically advertise the IP
              Prefixes within the rule via Magic Transit when the rule is triggered. Only
              available for users of Magic Transit.

          name: The name of the rule. Must be unique. Supports characters A-Z, a-z, 0-9,
              underscore (\\__), dash (-), period (.), and tilde (~). You can’t have a space in
              the rule name. Max 256 characters.

          type: MNM rule type.

          bandwidth_threshold: The number of bits per second for the rule. When this value is exceeded for the
              set duration, an alert notification is sent. Minimum of 1 and no maximum.

          duration: The amount of time that the rule threshold must be exceeded to send an alert
              notification. The final value must be equivalent to one of the following 8
              values ["1m","5m","10m","15m","20m","30m","45m","60m"].

          packet_threshold: The number of packets per second for the rule. When this value is exceeded for
              the set duration, an alert notification is sent. Minimum of 1 and no maximum.

          prefix_match: Prefix match type to be applied for a prefix auto advertisement when using an
              advanced_ddos rule.

          zscore_sensitivity: Level of sensitivity set for zscore rules.

          zscore_target: Target of the zscore rule analysis.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/mnm/rules", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "automatic_advertisement": automatic_advertisement,
                    "name": name,
                    "prefixes": prefixes,
                    "type": type,
                    "bandwidth_threshold": bandwidth_threshold,
                    "duration": duration,
                    "packet_threshold": packet_threshold,
                    "prefix_match": prefix_match,
                    "zscore_sensitivity": zscore_sensitivity,
                    "zscore_target": zscore_target,
                },
                rule_create_params.RuleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MagicNetworkMonitoringRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MagicNetworkMonitoringRule]], ResultWrapper[MagicNetworkMonitoringRule]),
        )

    async def update(
        self,
        *,
        account_id: str,
        automatic_advertisement: Optional[bool],
        name: str,
        prefixes: SequenceNotStr[str],
        type: Literal["threshold", "zscore", "advanced_ddos"],
        bandwidth_threshold: float | Omit = omit,
        duration: Literal["1m", "5m", "10m", "15m", "20m", "30m", "45m", "60m"] | Omit = omit,
        packet_threshold: float | Omit = omit,
        prefix_match: Optional[Literal["exact", "subnet", "supernet"]] | Omit = omit,
        zscore_sensitivity: Optional[Literal["low", "medium", "high"]] | Omit = omit,
        zscore_target: Optional[Literal["bits", "packets"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MagicNetworkMonitoringRule]:
        """
        Update network monitoring rules for account.

        Args:
          automatic_advertisement: Toggle on if you would like Cloudflare to automatically advertise the IP
              Prefixes within the rule via Magic Transit when the rule is triggered. Only
              available for users of Magic Transit.

          name: The name of the rule. Must be unique. Supports characters A-Z, a-z, 0-9,
              underscore (\\__), dash (-), period (.), and tilde (~). You can’t have a space in
              the rule name. Max 256 characters.

          type: MNM rule type.

          bandwidth_threshold: The number of bits per second for the rule. When this value is exceeded for the
              set duration, an alert notification is sent. Minimum of 1 and no maximum.

          duration: The amount of time that the rule threshold must be exceeded to send an alert
              notification. The final value must be equivalent to one of the following 8
              values ["1m","5m","10m","15m","20m","30m","45m","60m"].

          packet_threshold: The number of packets per second for the rule. When this value is exceeded for
              the set duration, an alert notification is sent. Minimum of 1 and no maximum.

          prefix_match: Prefix match type to be applied for a prefix auto advertisement when using an
              advanced_ddos rule.

          zscore_sensitivity: Level of sensitivity set for zscore rules.

          zscore_target: Target of the zscore rule analysis.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/mnm/rules", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "automatic_advertisement": automatic_advertisement,
                    "name": name,
                    "prefixes": prefixes,
                    "type": type,
                    "bandwidth_threshold": bandwidth_threshold,
                    "duration": duration,
                    "packet_threshold": packet_threshold,
                    "prefix_match": prefix_match,
                    "zscore_sensitivity": zscore_sensitivity,
                    "zscore_target": zscore_target,
                },
                rule_update_params.RuleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MagicNetworkMonitoringRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MagicNetworkMonitoringRule]], ResultWrapper[MagicNetworkMonitoringRule]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Optional[MagicNetworkMonitoringRule], AsyncSinglePage[Optional[MagicNetworkMonitoringRule]]]:
        """
        Lists network monitoring rules for account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/mnm/rules", account_id=account_id),
            page=AsyncSinglePage[Optional[MagicNetworkMonitoringRule]],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=MagicNetworkMonitoringRule,
        )

    async def delete(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MagicNetworkMonitoringRule]:
        """
        Delete a network monitoring rule for account.

        Args:
          rule_id: The id of the rule. Must be unique.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._delete(
            path_template("/accounts/{account_id}/mnm/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MagicNetworkMonitoringRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MagicNetworkMonitoringRule]], ResultWrapper[MagicNetworkMonitoringRule]),
        )

    async def edit(
        self,
        rule_id: str,
        *,
        account_id: str,
        automatic_advertisement: Optional[bool],
        name: str,
        prefixes: SequenceNotStr[str],
        type: Literal["threshold", "zscore", "advanced_ddos"],
        bandwidth_threshold: float | Omit = omit,
        duration: Literal["1m", "5m", "10m", "15m", "20m", "30m", "45m", "60m"] | Omit = omit,
        packet_threshold: float | Omit = omit,
        prefix_match: Optional[Literal["exact", "subnet", "supernet"]] | Omit = omit,
        zscore_sensitivity: Optional[Literal["low", "medium", "high"]] | Omit = omit,
        zscore_target: Optional[Literal["bits", "packets"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MagicNetworkMonitoringRule]:
        """
        Update a network monitoring rule for account.

        Args:
          rule_id: The id of the rule. Must be unique.

          automatic_advertisement: Toggle on if you would like Cloudflare to automatically advertise the IP
              Prefixes within the rule via Magic Transit when the rule is triggered. Only
              available for users of Magic Transit.

          name: The name of the rule. Must be unique. Supports characters A-Z, a-z, 0-9,
              underscore (\\__), dash (-), period (.), and tilde (~). You can’t have a space in
              the rule name. Max 256 characters.

          type: MNM rule type.

          bandwidth_threshold: The number of bits per second for the rule. When this value is exceeded for the
              set duration, an alert notification is sent. Minimum of 1 and no maximum.

          duration: The amount of time that the rule threshold must be exceeded to send an alert
              notification. The final value must be equivalent to one of the following 8
              values ["1m","5m","10m","15m","20m","30m","45m","60m"].

          packet_threshold: The number of packets per second for the rule. When this value is exceeded for
              the set duration, an alert notification is sent. Minimum of 1 and no maximum.

          prefix_match: Prefix match type to be applied for a prefix auto advertisement when using an
              advanced_ddos rule.

          zscore_sensitivity: Level of sensitivity set for zscore rules.

          zscore_target: Target of the zscore rule analysis.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._patch(
            path_template("/accounts/{account_id}/mnm/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            body=await async_maybe_transform(
                {
                    "automatic_advertisement": automatic_advertisement,
                    "name": name,
                    "prefixes": prefixes,
                    "type": type,
                    "bandwidth_threshold": bandwidth_threshold,
                    "duration": duration,
                    "packet_threshold": packet_threshold,
                    "prefix_match": prefix_match,
                    "zscore_sensitivity": zscore_sensitivity,
                    "zscore_target": zscore_target,
                },
                rule_edit_params.RuleEditParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MagicNetworkMonitoringRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MagicNetworkMonitoringRule]], ResultWrapper[MagicNetworkMonitoringRule]),
        )

    async def get(
        self,
        rule_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[MagicNetworkMonitoringRule]:
        """
        List a single network monitoring rule for account.

        Args:
          rule_id: The id of the rule. Must be unique.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not rule_id:
            raise ValueError(f"Expected a non-empty value for `rule_id` but received {rule_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/mnm/rules/{rule_id}", account_id=account_id, rule_id=rule_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[MagicNetworkMonitoringRule]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[MagicNetworkMonitoringRule]], ResultWrapper[MagicNetworkMonitoringRule]),
        )


class RulesResourceWithRawResponse:
    def __init__(self, rules: RulesResource) -> None:
        self._rules = rules

        self.create = to_raw_response_wrapper(
            rules.create,
        )
        self.update = to_raw_response_wrapper(
            rules.update,
        )
        self.list = to_raw_response_wrapper(
            rules.list,
        )
        self.delete = to_raw_response_wrapper(
            rules.delete,
        )
        self.edit = to_raw_response_wrapper(
            rules.edit,
        )
        self.get = to_raw_response_wrapper(
            rules.get,
        )

    @cached_property
    def advertisements(self) -> AdvertisementsResourceWithRawResponse:
        return AdvertisementsResourceWithRawResponse(self._rules.advertisements)


class AsyncRulesResourceWithRawResponse:
    def __init__(self, rules: AsyncRulesResource) -> None:
        self._rules = rules

        self.create = async_to_raw_response_wrapper(
            rules.create,
        )
        self.update = async_to_raw_response_wrapper(
            rules.update,
        )
        self.list = async_to_raw_response_wrapper(
            rules.list,
        )
        self.delete = async_to_raw_response_wrapper(
            rules.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            rules.edit,
        )
        self.get = async_to_raw_response_wrapper(
            rules.get,
        )

    @cached_property
    def advertisements(self) -> AsyncAdvertisementsResourceWithRawResponse:
        return AsyncAdvertisementsResourceWithRawResponse(self._rules.advertisements)


class RulesResourceWithStreamingResponse:
    def __init__(self, rules: RulesResource) -> None:
        self._rules = rules

        self.create = to_streamed_response_wrapper(
            rules.create,
        )
        self.update = to_streamed_response_wrapper(
            rules.update,
        )
        self.list = to_streamed_response_wrapper(
            rules.list,
        )
        self.delete = to_streamed_response_wrapper(
            rules.delete,
        )
        self.edit = to_streamed_response_wrapper(
            rules.edit,
        )
        self.get = to_streamed_response_wrapper(
            rules.get,
        )

    @cached_property
    def advertisements(self) -> AdvertisementsResourceWithStreamingResponse:
        return AdvertisementsResourceWithStreamingResponse(self._rules.advertisements)


class AsyncRulesResourceWithStreamingResponse:
    def __init__(self, rules: AsyncRulesResource) -> None:
        self._rules = rules

        self.create = async_to_streamed_response_wrapper(
            rules.create,
        )
        self.update = async_to_streamed_response_wrapper(
            rules.update,
        )
        self.list = async_to_streamed_response_wrapper(
            rules.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            rules.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            rules.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            rules.get,
        )

    @cached_property
    def advertisements(self) -> AsyncAdvertisementsResourceWithStreamingResponse:
        return AsyncAdvertisementsResourceWithStreamingResponse(self._rules.advertisements)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .scim import (
    SCIMResource,
    AsyncSCIMResource,
    SCIMResourceWithRawResponse,
    AsyncSCIMResourceWithRawResponse,
    SCIMResourceWithStreamingResponse,
    AsyncSCIMResourceWithStreamingResponse,
)
from .updates import (
    UpdatesResource,
    AsyncUpdatesResource,
    UpdatesResourceWithRawResponse,
    AsyncUpdatesResourceWithRawResponse,
    UpdatesResourceWithStreamingResponse,
    AsyncUpdatesResourceWithStreamingResponse,
)

__all__ = [
    "UpdatesResource",
    "AsyncUpdatesResource",
    "UpdatesResourceWithRawResponse",
    "AsyncUpdatesResourceWithRawResponse",
    "UpdatesResourceWithStreamingResponse",
    "AsyncUpdatesResourceWithStreamingResponse",
    "SCIMResource",
    "AsyncSCIMResource",
    "SCIMResourceWithRawResponse",
    "AsyncSCIMResourceWithRawResponse",
    "SCIMResourceWithStreamingResponse",
    "AsyncSCIMResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SubnetListParams"]


class SubnetListParams(TypedDict, total=False):
    account_id: Required[str]
    """Cloudflare account ID"""

    address_family: Literal["v4", "v6"]
    """If set, only include subnets in the given address family - `v4` or `v6`"""

    comment: str
    """If set, only list subnets with the given comment."""

    existed_at: str
    """
    If provided, include only resources that were created (and not deleted) before
    this time. URL encoded.
    """

    is_default_network: bool
    """If `true`, only include default subnets.

    If `false`, exclude default subnets subnets. If not set, all subnets will be
    included.
    """

    is_deleted: bool
    """If `true`, only include deleted subnets.

    If `false`, exclude deleted subnets. If not set, all subnets will be included.
    """

    name: str
    """If set, only list subnets with the given name"""

    network: str
    """If set, only list the subnet whose network exactly matches the given CIDR."""

    page: float
    """Page number of paginated results."""

    per_page: float
    """Number of results to display."""

    sort_order: Literal["asc", "desc"]
    """Sort order of the results.

    `asc` means oldest to newest, `desc` means newest to oldest. If not set, they
    will not be in any particular order.
    """

    subnet_types: Literal["cloudflare_source", "warp"]
    """If set, the types of subnets to include, separated by comma."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .issues import (
    IssuesResource,
    AsyncIssuesResource,
    IssuesResourceWithRawResponse,
    AsyncIssuesResourceWithRawResponse,
    IssuesResourceWithStreamingResponse,
    AsyncIssuesResourceWithStreamingResponse,
)
from ...._compat import cached_property
from .issue_types import (
    IssueTypesResource,
    AsyncIssueTypesResource,
    IssueTypesResourceWithRawResponse,
    AsyncIssueTypesResourceWithRawResponse,
    IssueTypesResourceWithStreamingResponse,
    AsyncIssueTypesResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["AttackSurfaceReportResource", "AsyncAttackSurfaceReportResource"]


class AttackSurfaceReportResource(SyncAPIResource):
    @cached_property
    def issue_types(self) -> IssueTypesResource:
        return IssueTypesResource(self._client)

    @cached_property
    def issues(self) -> IssuesResource:
        return IssuesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AttackSurfaceReportResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AttackSurfaceReportResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AttackSurfaceReportResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AttackSurfaceReportResourceWithStreamingResponse(self)


class AsyncAttackSurfaceReportResource(AsyncAPIResource):
    @cached_property
    def issue_types(self) -> AsyncIssueTypesResource:
        return AsyncIssueTypesResource(self._client)

    @cached_property
    def issues(self) -> AsyncIssuesResource:
        return AsyncIssuesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAttackSurfaceReportResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAttackSurfaceReportResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAttackSurfaceReportResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAttackSurfaceReportResourceWithStreamingResponse(self)


class AttackSurfaceReportResourceWithRawResponse:
    def __init__(self, attack_surface_report: AttackSurfaceReportResource) -> None:
        self._attack_surface_report = attack_surface_report

    @cached_property
    def issue_types(self) -> IssueTypesResourceWithRawResponse:
        return IssueTypesResourceWithRawResponse(self._attack_surface_report.issue_types)

    @cached_property
    def issues(self) -> IssuesResourceWithRawResponse:
        return IssuesResourceWithRawResponse(self._attack_surface_report.issues)


class AsyncAttackSurfaceReportResourceWithRawResponse:
    def __init__(self, attack_surface_report: AsyncAttackSurfaceReportResource) -> None:
        self._attack_surface_report = attack_surface_report

    @cached_property
    def issue_types(self) -> AsyncIssueTypesResourceWithRawResponse:
        return AsyncIssueTypesResourceWithRawResponse(self._attack_surface_report.issue_types)

    @cached_property
    def issues(self) -> AsyncIssuesResourceWithRawResponse:
        return AsyncIssuesResourceWithRawResponse(self._attack_surface_report.issues)


class AttackSurfaceReportResourceWithStreamingResponse:
    def __init__(self, attack_surface_report: AttackSurfaceReportResource) -> None:
        self._attack_surface_report = attack_surface_report

    @cached_property
    def issue_types(self) -> IssueTypesResourceWithStreamingResponse:
        return IssueTypesResourceWithStreamingResponse(self._attack_surface_report.issue_types)

    @cached_property
    def issues(self) -> IssuesResourceWithStreamingResponse:
        return IssuesResourceWithStreamingResponse(self._attack_surface_report.issues)


class AsyncAttackSurfaceReportResourceWithStreamingResponse:
    def __init__(self, attack_surface_report: AsyncAttackSurfaceReportResource) -> None:
        self._attack_surface_report = attack_surface_report

    @cached_property
    def issue_types(self) -> AsyncIssueTypesResourceWithStreamingResponse:
        return AsyncIssueTypesResourceWithStreamingResponse(self._attack_surface_report.issue_types)

    @cached_property
    def issues(self) -> AsyncIssuesResourceWithStreamingResponse:
        return AsyncIssuesResourceWithStreamingResponse(self._attack_surface_report.issues)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PipelineCreateV1Params"]


class PipelineCreateV1Params(TypedDict, total=False):
    account_id: Required[str]
    """Specifies the public ID of the account."""

    name: Required[str]
    """Specifies the name of the Pipeline."""

    sql: Required[str]
    """Specifies SQL for the Pipeline processing flow."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["VerificationEditParams"]


class VerificationEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    validation_method: Required[Literal["http", "cname", "txt", "email"]]
    """Desired validation method."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .mechanism_param import MechanismParam
from .policy_filter_param import PolicyFilterParam

__all__ = ["PolicyUpdateParams"]


class PolicyUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """The account id"""

    alert_interval: str
    """
    Optional specification of how often to re-alert from the same incident, not
    support on all alert types.
    """

    alert_type: Literal[
        "abuse_report_alert",
        "access_custom_certificate_expiration_type",
        "advanced_ddos_attack_l4_alert",
        "advanced_ddos_attack_l7_alert",
        "advanced_http_alert_error",
        "bgp_hijack_notification",
        "billing_usage_alert",
        "block_notification_block_removed",
        "block_notification_new_block",
        "block_notification_review_rejected",
        "bot_traffic_basic_alert",
        "brand_protection_alert",
        "brand_protection_digest",
        "clickhouse_alert_fw_anomaly",
        "clickhouse_alert_fw_ent_anomaly",
        "cloudforce_one_request_notification",
        "cni_maintenance_notification",
        "custom_analytics",
        "custom_bot_detection_alert",
        "custom_ssl_certificate_event_type",
        "dedicated_ssl_certificate_event_type",
        "device_connectivity_anomaly_alert",
        "dos_attack_l4",
        "dos_attack_l7",
        "expiring_service_token_alert",
        "failing_logpush_job_disabled_alert",
        "fbm_auto_advertisement",
        "fbm_dosd_attack",
        "fbm_volumetric_attack",
        "health_check_status_notification",
        "hostname_aop_custom_certificate_expiration_type",
        "http_alert_edge_error",
        "http_alert_origin_error",
        "image_notification",
        "image_resizing_notification",
        "incident_alert",
        "load_balancing_health_alert",
        "load_balancing_pool_enablement_alert",
        "logo_match_alert",
        "magic_tunnel_health_check_event",
        "magic_wan_tunnel_health",
        "maintenance_event_notification",
        "mtls_certificate_store_certificate_expiration_type",
        "pages_event_alert",
        "radar_notification",
        "real_origin_monitoring",
        "scriptmonitor_alert_new_code_change_detections",
        "scriptmonitor_alert_new_hosts",
        "scriptmonitor_alert_new_malicious_hosts",
        "scriptmonitor_alert_new_malicious_scripts",
        "scriptmonitor_alert_new_malicious_url",
        "scriptmonitor_alert_new_max_length_resource_url",
        "scriptmonitor_alert_new_resources",
        "secondary_dns_all_primaries_failing",
        "secondary_dns_primaries_failing",
        "secondary_dns_warning",
        "secondary_dns_zone_successfully_updated",
        "secondary_dns_zone_validation_warning",
        "security_insights_alert",
        "sentinel_alert",
        "stream_live_notifications",
        "synthetic_test_latency_alert",
        "synthetic_test_low_availability_alert",
        "traffic_anomalies_alert",
        "tunnel_health_event",
        "tunnel_update_event",
        "universal_ssl_event_type",
        "web_analytics_metrics_update",
        "zone_aop_custom_certificate_expiration_type",
    ]
    """Refers to which event will trigger a Notification dispatch.

    You can use the endpoint to get available alert types which then will give you a
    list of possible values.
    """

    description: str
    """Optional description for the Notification policy."""

    enabled: bool
    """Whether or not the Notification policy is enabled."""

    filters: PolicyFilterParam
    """
    Optional filters that allow you to be alerted only on a subset of events for
    that alert type based on some criteria. This is only available for select alert
    types. See alert type documentation for more details.
    """

    mechanisms: MechanismParam
    """List of IDs that will be used when dispatching a notification.

    IDs for email type will be the email address.
    """

    name: str
    """Name of the policy."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["HistoryListParams"]


class HistoryListParams(TypedDict, total=False):
    account_id: Required[str]
    """The account id"""

    before: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Limit the returned results to history records older than the specified date.

    This must be a timestamp that conforms to RFC3339.
    """

    page: float
    """Page number of paginated results."""

    per_page: float
    """Number of items per page."""

    since: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Limit the returned results to history records newer than the specified date.

    This must be a timestamp that conforms to RFC3339.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from ..rules.lists.list_cursor import ListCursor

__all__ = ["EnvironmentCreateResponse", "Environment"]


class Environment(BaseModel):
    expression: str

    locked_on_deployment: Optional[bool] = None

    name: str

    position: ListCursor

    ref: str

    version: Optional[int] = None

    http_application_id: Optional[str] = None


class EnvironmentCreateResponse(BaseModel):
    environments: List[Environment]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from .fields import (
    FieldsResource,
    AsyncFieldsResource,
    FieldsResourceWithRawResponse,
    AsyncFieldsResourceWithRawResponse,
    FieldsResourceWithStreamingResponse,
    AsyncFieldsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["DatasetsResource", "AsyncDatasetsResource"]


class DatasetsResource(SyncAPIResource):
    @cached_property
    def fields(self) -> FieldsResource:
        return FieldsResource(self._client)

    @cached_property
    def jobs(self) -> JobsResource:
        return JobsResource(self._client)

    @cached_property
    def with_raw_response(self) -> DatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DatasetsResourceWithStreamingResponse(self)


class AsyncDatasetsResource(AsyncAPIResource):
    @cached_property
    def fields(self) -> AsyncFieldsResource:
        return AsyncFieldsResource(self._client)

    @cached_property
    def jobs(self) -> AsyncJobsResource:
        return AsyncJobsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDatasetsResourceWithStreamingResponse(self)


class DatasetsResourceWithRawResponse:
    def __init__(self, datasets: DatasetsResource) -> None:
        self._datasets = datasets

    @cached_property
    def fields(self) -> FieldsResourceWithRawResponse:
        return FieldsResourceWithRawResponse(self._datasets.fields)

    @cached_property
    def jobs(self) -> JobsResourceWithRawResponse:
        return JobsResourceWithRawResponse(self._datasets.jobs)


class AsyncDatasetsResourceWithRawResponse:
    def __init__(self, datasets: AsyncDatasetsResource) -> None:
        self._datasets = datasets

    @cached_property
    def fields(self) -> AsyncFieldsResourceWithRawResponse:
        return AsyncFieldsResourceWithRawResponse(self._datasets.fields)

    @cached_property
    def jobs(self) -> AsyncJobsResourceWithRawResponse:
        return AsyncJobsResourceWithRawResponse(self._datasets.jobs)


class DatasetsResourceWithStreamingResponse:
    def __init__(self, datasets: DatasetsResource) -> None:
        self._datasets = datasets

    @cached_property
    def fields(self) -> FieldsResourceWithStreamingResponse:
        return FieldsResourceWithStreamingResponse(self._datasets.fields)

    @cached_property
    def jobs(self) -> JobsResourceWithStreamingResponse:
        return JobsResourceWithStreamingResponse(self._datasets.jobs)


class AsyncDatasetsResourceWithStreamingResponse:
    def __init__(self, datasets: AsyncDatasetsResource) -> None:
        self._datasets = datasets

    @cached_property
    def fields(self) -> AsyncFieldsResourceWithStreamingResponse:
        return AsyncFieldsResourceWithStreamingResponse(self._datasets.fields)

    @cached_property
    def jobs(self) -> AsyncJobsResourceWithStreamingResponse:
        return AsyncJobsResourceWithStreamingResponse(self._datasets.jobs)
